package be.iminds.ilabt.jfed.restapi;

import be.iminds.ilabt.jfed.restapi.dao.DebugDataDao;
import be.iminds.ilabt.jfed.restapi.dao.ExperimentDao;
import be.iminds.ilabt.jfed.restapi.dao.SpeaksForCredentialDao;
import be.iminds.ilabt.jfed.restapi.debugdata.DebugDataManager;
import be.iminds.ilabt.jfed.restapi.dropwizard.JFedRestApiDebugDataAppenderFactory;
import be.iminds.ilabt.jfed.restapi.dropwizard.JFedRestApiSiftingAppenderFactory;
import be.iminds.ilabt.jfed.restapi.resource.DebugDataResource;
import be.iminds.ilabt.jfed.restapi.resource.ExperimentResource;
import be.iminds.ilabt.jfed.restapi.resource.SpeaksForCredentialResource;
import be.iminds.ilabt.jfed.restapi.resource.UserResource;
import com.fasterxml.jackson.databind.DeserializationFeature;
import io.dropwizard.Application;
import io.dropwizard.assets.AssetsBundle;
import io.dropwizard.db.DataSourceFactory;
import io.dropwizard.jdbi.DBIFactory;
import io.dropwizard.logging.AppenderFactory;
import io.dropwizard.logging.DefaultLoggingFactory;
import io.dropwizard.migrations.MigrationsBundle;
import io.dropwizard.setup.Bootstrap;
import io.dropwizard.setup.Environment;
import javafx.embed.swing.JFXPanel;
import org.eclipse.jetty.servlets.CrossOriginFilter;
import org.skife.jdbi.v2.DBI;

import javax.servlet.DispatcherType;
import javax.servlet.FilterRegistration;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

public class JFedRestApiApplication extends Application<JFedRestApiConfiguration> {
    public static void main(String[] args) throws Exception {
        new JFedRestApiApplication().run(args);
    }

    @Override
    public String getName() {
        return "f-interop-bridge-webapi-service";
    }

    @Override
    public void initialize(Bootstrap<JFedRestApiConfiguration> bootstrap) {
        bootstrap.addBundle(new AssetsBundle("/website", "/", "index.html"));

        bootstrap.addBundle(new MigrationsBundle<JFedRestApiConfiguration>() {
            @Override
            public DataSourceFactory getDataSourceFactory(JFedRestApiConfiguration configuration) {
                return configuration.getDataSourceFactory();
            }
        });

        //swagger is not that usefull, and is generating too much warnings
//        bootstrap.addBundle(new SwaggerBundle<JFedRestApiConfiguration>() {
//            @Override
//            protected SwaggerBundleConfiguration getSwaggerBundleConfiguration(JFedRestApiConfiguration configuration) {
//                return configuration.getSwaggerBundleConfiguration();
//            }
//        });
    }


    @Override
    public void run(JFedRestApiConfiguration configuration,
                    Environment environment) {


        //force initialization of JavaFX  (we don't need the GUI parts, bu we do use the model parts)
        try {
            new JFXPanel();
        }catch(RuntimeException ex){
            throw new RuntimeException("Could not successfully load JavaFX toolkit", ex);
        }

        final Map<String, JFedRestApiSiftingAppenderFactory> jFedRestApiAppenderFactoryByMDCKey = new HashMap<>();
        JFedRestApiDebugDataAppenderFactory jFedRestApiDebugDataAppenderFactory = null;
        final DefaultLoggingFactory loggingFactory = (DefaultLoggingFactory) configuration.getLoggingFactory();
        for (AppenderFactory appenderFactory : loggingFactory.getAppenders()) {
            if (appenderFactory instanceof JFedRestApiSiftingAppenderFactory) {
                JFedRestApiSiftingAppenderFactory jFedRestApiAppenderFactory = (JFedRestApiSiftingAppenderFactory) appenderFactory;
                jFedRestApiAppenderFactoryByMDCKey.put(jFedRestApiAppenderFactory.getMdcKey(), jFedRestApiAppenderFactory);
            }
            if (appenderFactory instanceof JFedRestApiDebugDataAppenderFactory) {
                jFedRestApiDebugDataAppenderFactory = (JFedRestApiDebugDataAppenderFactory) appenderFactory;
            }
        }

        if (jFedRestApiDebugDataAppenderFactory == null) {
            throw new RuntimeException("Must configure appender jFedRestApiDebugData in dropwizard server confg yml");
        }

        final DBIFactory factory = new DBIFactory();
        final DBI jdbi = factory.build(environment, configuration.getDataSourceFactory(), "postgresql");

        final ExperimentDao experimentDao = jdbi.onDemand(ExperimentDao.class);
        final DebugDataDao debugDataDao = jdbi.onDemand(DebugDataDao.class);
        final SpeaksForCredentialDao speaksForCredentialDao = jdbi.onDemand(SpeaksForCredentialDao.class);

        final DebugDataManager debugDataManager = new DebugDataManager(jFedRestApiDebugDataAppenderFactory.getAppender(), debugDataDao);

        final ExperimentResource mainResource = new ExperimentResource(
                configuration, experimentDao, speaksForCredentialDao, debugDataManager, jFedRestApiAppenderFactoryByMDCKey);
        final DebugDataResource debugDataResource = new DebugDataResource(debugDataManager, debugDataDao, configuration);

        final SpeaksForCredentialResource speaksForCredentialResource = new SpeaksForCredentialResource(configuration, speaksForCredentialDao);
        final UserResource userResource = new UserResource(configuration, speaksForCredentialDao);

        //be forgiving when clients send unknown fields
        environment.getObjectMapper().configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, true);

        environment.jersey().register(mainResource);
        environment.jersey().register(debugDataResource);
        environment.jersey().register(speaksForCredentialResource);
        environment.jersey().register(userResource);

        //enable CORS (Cross-origin resource sharing) support
        System.setProperty("sun.net.http.allowRestrictedHeaders", "true");
        final FilterRegistration.Dynamic filter = environment.servlets().addFilter("CORSFilter", CrossOriginFilter.class);
        //filter.addMappingForUrlPatterns(EnumSet.of(DispatcherType.REQUEST), false, environment.getApplicationContext().getContextPath() + "*");
        // Configure CORS parameters
        filter.setInitParameter("allowedOrigins", "*");
        filter.setInitParameter("allowedHeaders", "X-Requested-With,Content-Type,Accept,Origin");
        filter.setInitParameter("allowedMethods", "OPTIONS,GET,HEAD,POST,PUT,DELETE");
        // Add URL mapping
        filter.addMappingForUrlPatterns(EnumSet.allOf(DispatcherType.class), true, "/*");
    }
}
