package be.iminds.ilabt.jfed.restapi.dao;

import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.restapi.json.SpeaksForCredential;
import be.iminds.ilabt.jfed.util.GeniUrn;
import org.skife.jdbi.v2.sqlobject.*;
import org.skife.jdbi.v2.sqlobject.customizers.RegisterMapper;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

@RegisterMapper(SpeaksForCredentialMapper.class)
public abstract class SpeaksForCredentialDao {
    @SqlQuery("SELECT speaker,speaking_for,expire,credential " +
            " FROM speaksfor " +
            " WHERE speaker=:serviceUser AND speaking_for=:spokenForUser AND expire >= NOW()")
    public abstract SpeaksForCredential find(@NotNull @BindGeniUrn("serviceUser") GeniUrn serviceUser, @NotNull @BindGeniUrn("spokenForUser") GeniUrn spokenForUser);

    //this requires postgress >= 9.5 due to "ON CONFLICT"
    @SqlUpdate("INSERT INTO speaksfor (speaker,speaking_for,expire,credential) " +
            "VALUES (:serviceUser, :spokenForUser, :expiresDate, :speaksForCredentialXml) "+
            "ON CONFLICT (speaker, speaking_for) " +
            "DO UPDATE SET expire=:expiresDate, credential=:speaksForCredentialXml " +
            "WHERE speaksfor.expire <= :expiresDate")
    public abstract void upsert(@NotNull @BindGeniUrn("serviceUser") GeniUrn serviceUser,
                @NotNull @BindGeniUrn("spokenForUser") GeniUrn spokenForUser,
                @NotNull @Bind("expiresDate") Date expiresDate,
                @NotNull @Bind("speaksForCredentialXml") String speaksForCredentialXml);

    @Transaction
    public void upsert(SpeaksForCredential speaksForCredential) {
        upsert(speaksForCredential.getSpeaker(),
                speaksForCredential.getSpeakingFor(),
                speaksForCredential.getExpire(),
                speaksForCredential.getCredentialXml());
    }
}
